﻿using Http.Core;
using Http.Core.Extensions;
using PCUT.Entities;
using PCUT.Entities.ApiResponse;
using PCUT.Models;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using static Http.Core.Constants.HttpConstants;

namespace PCUT.ViewModels
{
    public class LogViewModel : NotificationBase
    {
        public PaginationViewModel Pagination { get; } = new PaginationViewModel(withSearch: false);
        public ObservableCollection<Log> Logs { get; }

        public LogViewModel()
        {
            Logs = new ObservableCollection<Log>();

            Pagination.PageChanged += OnPageChanged;
        }

        private async void OnPageChanged(object sender, PageChangedEventArgs args)
        {
            await LoadLogsAsync(args.Page, args.PageSize);
        }

        public async Task LoadLogsAsync(int page, int pageSize)
        {
            var url = CreateFilterUrl(page, pageSize);
            using (var httpClient = HttpClientFactory.CreateClient(ClientNames.ApiClient))
            {
                try
                {
                    var response = await httpClient.GetAsync(url);
                    if (response.IsSuccessStatusCode)
                    {
                        var apiResponse = await response.DeserializeObjectAsync<PaginationResponse<IEnumerable<Log>>>();
                        var data = apiResponse.Data;
                        Pagination.Page = apiResponse.Pagination.Page;
                        Pagination.TotalPage = apiResponse.Pagination.TotalPages;
                        Pagination.TotalRecords = apiResponse.Pagination.TotalRecords;

                        Logs.Clear();
                        foreach (var log in data)
                            Logs.Add(log);
                    }
                }
                catch (Exception ex)
                {

                }
            }
        }

        private string CreateFilterUrl(int page, int pageSize/*, string searchText*/)
        {
            var builder = PathBuilder.FromRouteTemplate(Api.Log)
                                     .AddQuery("page", page.ToString())
                                     .AddQuery("pageSize", pageSize.ToString());
            //if (!string.IsNullOrEmpty(searchText))
            //{
            //    var filter = new StringBuilder().AppendFilter("username", "cn", searchText).BuildFilter().ToString();
            //    builder.AddQuery("filter", filter);
            //}
            return builder.Build();
        }
    }
}
